@@ -116,6 +116,8 @@ class Ngpf(object):
         if len(ixn_device_group) == 0:
             ixn_device_group.add(**args)[-1]
         else:
+            ixn_ng = ixn_device_group.NetworkGroup
+            self._api._remove(ixn_ng, [])
             self._update(ixn_device_group, **args)
         self._config_proto_stack(ixn_device_group, device, ixn_device_group)
     
@@ -269,11 +271,6 @@ class Ngpf(object):
     def _bgp_route_builder(self, ixn_dg, ixn_bgp, bgp):
         bgpv4_routes = bgp.bgpv4_routes
         bgpv6_routes = bgp.bgpv6_routes
-        route_ranges = []
-        route_ranges.extend(bgpv4_routes)
-        route_ranges.extend(bgpv6_routes)
-        if len(route_ranges) > 0:
-            self._api._remove(ixn_dg.NetworkGroup, route_ranges)
         if len(bgpv4_routes) > 0:
             for route_range in bgpv4_routes:
                 self._configure_bgpv4_route(ixn_dg,
@@ -430,10 +427,8 @@ class Ngpf(object):
     
     def _config_bgp_as_path(self, as_path, ixn_bgp_property):
         as_path_segments = as_path.as_path_segments
-        if as_path.as_set_mode is None or len(
-                as_path_segments) == 0:
-            ixn_bgp_property.EnableAsPathSegments.Single(False)
-        else:
+        if as_path.as_set_mode is not None or len(
+                as_path_segments) > 0:
             ixn_bgp_property.EnableAsPathSegments.Single(True)
             self._configure_pattern(ixn_bgp_property.AsSetMode,
                                     as_path.as_set_mode, Ngpf._BGP_AS_MODE)
